@@ -71,12 +71,6 @@ factory.doc['props']['required_class'] = \
 """CSS-class to put on in case if required condition was not met.
 """
 
-# XXX: remove. Controlled by form.novalidate
-factory.defaults['html5required'] = True
-factory.doc['props']['html5required'] = \
-"""Flag whether HTML5 required attribute should be rendered.
-"""
-
 factory.defaults['size'] = None
 factory.doc['props']['size'] = \
 """Allowed input size.
@@ -93,6 +87,13 @@ factory.doc['props']['required_class_default'] = \
 class was given.
 """
 
+factory.defaults['template'] = '%s'
+factory.doc['props']['template'] = \
+"""Format string with pythons a built-in string format template. If a callable 
+is given it will be used instead and is called with ``widget`` and ``data`` as 
+parameters.  
+"""
+
 
 ###############################################################################
 # generic
@@ -142,7 +143,6 @@ def generic_required_extractor(widget, data):
 def input_generic_renderer(widget, data):
     """Generic HTML ``input`` tag render.
     """
-    tag = data.tag
     input_attrs = {
         'type': widget.attrs['type'],
         'value': fetch_value(widget, data),
@@ -155,25 +155,49 @@ def input_generic_renderer(widget, data):
         'disabled': bool(widget.attrs.get('disabled')) and 'disabled' or None,
         'autocomplete': widget.attrs.get('autocomplete'),
     }
-    # XXX: remove. Controlled by form.novalidate
-    html5required = widget.attrs['html5required']
-    if html5required:
-        input_attrs['required'] = \
-            widget.attrs.get('required') and 'required' or None
+    input_attrs['required'] = \
+        widget.attrs.get('required') and 'required' or None
     if widget.attrs['type'] in ['range', 'number']:
         input_attrs['min'] = widget.attrs.get('min') or None
         input_attrs['max'] = widget.attrs.get('min') or None
         input_attrs['step'] = widget.attrs.get('step') or None
-    return tag('input', **input_attrs)
+    return data.tag('input', **input_attrs)
+
+
+@managedprops('template', 'class')
+def generic_display_renderer(widget, data):
+    """Generic display renderer to render a value.
+    """
+    if callable(widget.attrs['template']):
+        content = widget.attrs['template'](widget, data)
+    else:
+        value = fetch_value(widget, data)
+        if value is None:
+            value = u''
+        content = widget.attrs['template'] % value
+    attrs = {
+        'id': cssid(widget, 'display'),
+        'class_': 'display-%s' % widget.attrs['class'] or 'generic'
+    }
+    return data.tag('div', content, **attrs)
+
+
+def empty_display_renderer(widget, data):
+    """Display renderer which renders an empty string.
+    """
+    return u''
 
 
 ###############################################################################
 # text
 ###############################################################################
 
-factory.register('text', 
-                 [generic_extractor, generic_required_extractor], 
-                 [input_generic_renderer])
+factory.register(
+    'text', 
+    extractors=[generic_extractor, generic_required_extractor], 
+    edit_renderers=[input_generic_renderer],
+    display_renderers=[generic_display_renderer])
+
 factory.doc['widget']['text'] = \
 """Text input widget.
 """
@@ -199,9 +223,12 @@ factory.doc['props']['text.disabled'] = \
 # hidden
 ###############################################################################
 
-factory.register('hidden', 
-                 [generic_extractor], 
-                 [input_generic_renderer])
+factory.register(
+     'hidden', 
+     extractors=[generic_extractor], 
+     edit_renderers=[input_generic_renderer],
+     display_renderers=[empty_display_renderer])
+
 factory.doc['widget']['hidden'] = \
 """Hidden input widget.
 """
@@ -240,9 +267,12 @@ def input_proxy_renderer(widget, data):
     return tag('input', **input_attrs)
 
 
-factory.register('proxy', 
-                 [generic_extractor], 
-                 [input_proxy_renderer])
+factory.register(
+    'proxy', 
+    extractors=[generic_extractor], 
+    edit_renderers=[input_proxy_renderer],
+    display_renderers=[empty_display_renderer])
+
 factory.doc['widget']['proxy'] = \
 """Used to pass hidden arguments out of form namespace.
 """
@@ -278,9 +308,12 @@ def textarea_renderer(widget, data):
     return tag('textarea', value, **area_attrs)
 
 
-factory.register('textarea', 
-                 [generic_extractor, generic_required_extractor], 
-                 [textarea_renderer])
+factory.register(
+    'textarea', 
+    extractors=[generic_extractor, generic_required_extractor], 
+    edit_renderers=[textarea_renderer],
+    display_renderers=[generic_display_renderer])
+
 factory.doc['widget']['textarea'] = \
 """HTML textarea widget.
 """
@@ -355,6 +388,7 @@ def ascii_extractor(widget, data):
         raise ExtractionError(u'Input contains illegal characters.')
     return val
 
+
 LOWER_CASE_RE = '(?=.*[a-z])'
 UPPER_CASE_RE = '(?=.*[A-Z])'
 DIGIT_RE = '(?=.*[\d])'
@@ -406,20 +440,21 @@ def password_extractor(widget, data):
     return val
 
 
+def _pwd_value(widget, data):
+    if data.extracted is not UNSET:
+        return data.extracted
+    if data.value is not UNSET \
+      and data.value is not None:
+        return PASSWORD_NOCHANGE_VALUE
+    return widget.attrs['default']
+
 @managedprops('size', 'disabled', 'placeholder', 'autofocus', 'required',
               *css_managed_props)
-def password_renderer(widget, data):
+def password_edit_renderer(widget, data):
     """Render password widget.
     """
     tag = data.tag
-    def pwd_value(widget, data):
-        if data.extracted is not UNSET:
-            return data.extracted
-        if data.value is not UNSET \
-          and data.value is not None:
-            return PASSWORD_NOCHANGE_VALUE
-        return widget.attrs['default']
-    value = pwd_value(widget, data)
+    value = _pwd_value(widget, data)
     input_attrs = {
         'type': 'password',
         'value':  value,
@@ -431,19 +466,25 @@ def password_renderer(widget, data):
         'autofocus': widget.attrs.get('autofocus') and 'autofocus' or None,      
         'disabled': widget.attrs.get('disabled'),
     }
-    # XXX: remove. Controlled form.novalidate instead
-    html5required = widget.attrs['html5required']
-    if html5required:
-        input_attrs['required'] = \
-            widget.attrs.get('required') and 'required' or None
+    input_attrs['required'] = \
+        widget.attrs.get('required') and 'required' or None
     return tag('input', **input_attrs)
 
+@managedprops('displayplaceholder')
+def password_display_renderer(widget, data):
+    value = _pwd_value(widget, data)
+    if value == PASSWORD_NOCHANGE_VALUE:
+        return widget.attrs['displayplaceholder']
+    return u''
+
+
+factory.register(
+    'password', 
+    extractors=[generic_extractor, generic_required_extractor,
+                minlength_extractor, ascii_extractor, password_extractor],
+    edit_renderers=[password_edit_renderer],
+    display_renderers=[password_display_renderer])
 
-factory.register('password', 
-                 [generic_extractor, generic_required_extractor,
-                  minlength_extractor, ascii_extractor, password_extractor],
-                 [password_renderer],
-                 [])
 factory.doc['widget']['password'] = \
 """Password widget.
 
@@ -456,6 +497,7 @@ password extraction.
 factory.defaults['password.required_class'] = 'required'
 
 factory.defaults['password.default'] = ''
+
 factory.defaults['password.class'] = 'password'
 
 factory.defaults['password.minlength'] = -1
@@ -472,7 +514,6 @@ factory.doc['props']['password.ascii'] = \
 """Flag  ascii check should performed.
 """
 
-factory.defaults['password.strength'] = 'strength'
 factory.defaults['password.strength'] = -1
 factory.doc['props']['password.strength'] = \
 """Integer value <= 4. Define how many rules must apply to consider a password
@@ -483,6 +524,10 @@ factory.defaults['weak_password_message'] = u'Password too weak'
 factory.doc['props']['password.strength'] = \
 """Message shown if password is not strong enough.
 """
+factory.defaults['password.displayplaceholder'] = u'*'*8
+factory.doc['props']['password.displayplaceholder'] = \
+"""Placeholder shown in display mode if password was set. 
+"""
 
 
 ###############################################################################
@@ -490,7 +535,7 @@ factory.doc['props']['password.strength'] = \
 ###############################################################################
 
 @managedprops('format')
-def input_checkbox_extractor(widget, data):
+def checkbox_extractor(widget, data):
     """Extracts data from a single input with type checkbox.
     """
     if '%s-exists' % widget.dottedpath not in data.request:
@@ -504,7 +549,7 @@ def input_checkbox_extractor(widget, data):
 
 
 @managedprops('format', 'disabled', *css_managed_props)
-def input_checkbox_renderer(widget, data):
+def checkbox_edit_renderer(widget, data):
     tag = data.tag
     value = fetch_value(widget, data)
     input_attrs = {
@@ -528,19 +573,55 @@ def input_checkbox_renderer(widget, data):
     exists_marker = tag('input', **input_attrs)
     return checkbox + exists_marker
 
+@managedprops('bool', 'vocabulary')
+def checkbox_display_renderer(widget, data):
+    """Generic display renderer to render a value.
+    """
+    value = fetch_value(widget, data)
+    if widget.attrs['format'] == 'string' and bool(value):
+        content = value
+    else:
+        vocab = dict(vocabulary(widget.attrs.get('vocabulary', [])))
+        content = vocab[bool(value)]
+
+    attrs = {
+        'id': cssid(widget, 'display'),
+        'class_': 'display-%s' % widget.attrs['class'] or 'generic'
+    }
+    return data.tag('div', content, **attrs)
+
+
+factory.register(
+    'checkbox', 
+    extractors=[checkbox_extractor, generic_required_extractor], 
+    edit_renderers=[checkbox_edit_renderer],
+    display_renderers=[checkbox_display_renderer])
 
 factory.doc['widget']['checkbox'] = """\
 Single checkbox.
 """
+
 factory.defaults['checkbox.default'] = False
+
 factory.defaults['checkbox.format'] = 'bool'
 factory.doc['props']['checkbox.format'] = """\
 Data-type of the extracted value. One out of ``bool`` or ``string``. 
 """
+
+factory.defaults['checkbox.format'] = 'bool'
+factory.doc['props']['checkbox.format'] = """\
+Data-type of the extracted value. One out of ``bool`` or ``string``. 
+"""
+
+factory.defaults['checkbox.vocabulary'] = { True:'yes', False: 'no', 
+                                            UNSET: 'not set' }
+factory.doc['props']['checkbox.vocabulary'] = """\
+In display mode and if ```bool``` is set to ```True``` this mapping will be used
+for display of the value. Expected keys are ```True```, ```False``` and 
+```UNSET```.
+"""
+
 factory.defaults['checkbox.required_class'] = 'required'
-factory.register('checkbox', 
-                 [input_checkbox_extractor, generic_required_extractor], 
-                 [input_checkbox_renderer])
 
 
 ###############################################################################
@@ -587,13 +668,12 @@ def select_exists_marker(widget, data):
     return tag('input', **attrs)
 
 
-@managedprops('format', 'vocabulary', 'multivalued', 'disabled', *css_managed_props)
-def select_renderer(widget, data):
+@managedprops('format', 'vocabulary', 'multivalued', 'disabled', 
+              'listing_label_position', 'listing_tag', 
+              *css_managed_props)
+def select_edit_renderer(widget, data):
     tag = data.tag
     value = fetch_value(widget, data)
-    # Seem to be never called... keep a while
-    #if value is UNSET:
-    #    value = []
     if isinstance(value, basestring) or not hasattr(value, '__iter__'):
         value = [value]
     if not widget.attrs['multivalued'] and len(value) > 1:
@@ -633,6 +713,9 @@ def select_renderer(widget, data):
         return rendered
     else:
         tags = []
+        label_pos = widget.attrs['listing_label_position']
+        listing_tag = widget.attrs['listing_tag']
+        item_tag = listing_tag == 'div' and 'div' or 'li'
         for key, term in vocabulary(widget.attrs.get('vocabulary', [])):
             if widget.attrs['multivalued']:
                 tagtype = 'checkbox'
@@ -649,35 +732,77 @@ def select_renderer(widget, data):
             if (disabled and disabled is not True and key in disabled) \
                or disabled is True:
                 attrs['disabled'] = 'disabled'                                 
+            
             input = tag('input', **attrs)
-            text = tag('label', term, for_=attrs['id'])
-            tags.append(tag('div', input, text,
+            if label_pos == 'inner':
+                item = tag('label', term, input, for_=attrs['id'])
+            elif label_pos == 'after':
+                item = input + tag('label', term, for_=attrs['id'])
+            else:
+                item = tag('label', term, for_=attrs['id']) + input
+            tags.append(tag(item_tag, item,
                             **{'id': cssid(widget, tagtype, key)}))
-        return select_exists_marker(widget, data) + u''.join(tags)
+        return select_exists_marker(widget, data) + \
+            tag(listing_tag, *tags, **{'id': cssid(widget, tagtype, 'wrapper')})
 
 
+@managedprops('template', 'class', 'multivalued')
+def select_display_renderer(widget, data):
+    value = fetch_value(widget, data)
+    if not widget.attrs['multivalued'] or not value:
+        return generic_display_renderer(widget, data)
+    attrs = {
+        'id': cssid(widget, 'display'),
+        'class_': 'display-%s' % widget.attrs['class'] or 'generic'
+    }
+    content = u''
+    vocab = dict(vocabulary(widget.attrs.get('vocabulary', [])))
+    for key in value:
+        content += data.tag('li', vocab[key])
+    return data.tag('ul', content, **attrs)
+        
+
+factory.register(
+    'select',
+    extractors=[select_extractor, generic_required_extractor], 
+    edit_renderers=[select_edit_renderer],
+    display_renderers=[select_display_renderer])
+
 factory.doc['widget']['select'] = """\
 Selection Widget. Single selection as dropdown or radio-buttons. Multiple 
 selection as selection-list or as checkboxes. 
-"""        
+"""
+    
 factory.defaults['select.multivalued'] = None
+
 factory.defaults['select.default'] = []
+
 factory.defaults['select.format'] = 'block'
+
+factory.defaults['select.listing_tag'] = 'div'
+factory.doc['props']['select.listing_tag'] = """\
+Desired rendering tag for selection if selection format is 'single'. Valid
+values are 'div' and 'ul'.
+"""
+
+factory.defaults['select.listing_label_position'] = 'before'
+factory.doc['props']['select.listing_label_position'] = """\
+Label position if format is 'single'. Behaves the same way as label widget
+position property.
+"""
+
 factory.doc['props']['select.vocabulary'] = """\
 Vocabulary to be used for the selection list. Expects a dict-like or an iterable 
 or a callable which returns one of both first. An iterable can consist out of 
 strings or out of tuples with ``(key, value)``.   
 """
+
 factory.doc['props']['select.disabled'] = """\
 Disables the whole widget or single selections. To disable the whole widget just
 set the value to 'True'. To disable single selection pass a iterable of keys to 
 disable, i.e. ``['foo', 'baz']``. Defaults to False.
 """
 
-factory.register('select',
-                 [select_extractor, generic_required_extractor], 
-                 [select_renderer])
-
 
 ###############################################################################
 # file
@@ -741,23 +866,27 @@ def file_options_renderer(widget, data):
     return data.rendered + u''.join(tags)
 
 
+factory.register(
+    'file',
+    extractors=[file_extracor, generic_required_extractor],
+    edit_renderers=[input_file_renderer, file_options_renderer])
+
 factory.doc['widget']['file'] = """\
 A basic file upload widget.
 """
+
 factory.defaults['file.multivalued'] = False
+
 factory.defaults['file.accept'] = None
 factory.doc['props']['file.accept'] = """\
 Content type sto accept.
 """
 
 factory.defaults['file.vocabulary'] = [
-    ('keep', 'Keep Existing file'),
-    ('replace', 'Replace existing file'),
-    ('delete', 'Delete existing file'),
+    ('keep', u'Keep Existing file'),
+    ('replace', u'Replace existing file'),
+    ('delete', u'Delete existing file'),
 ]
-factory.register('file',
-                 [file_extracor, generic_required_extractor],
-                 [input_file_renderer, file_options_renderer])
 
 
 ###############################################################################
@@ -777,9 +906,15 @@ def submit_renderer(widget, data):
     return tag('input', **input_attrs)
 
 
+factory.register(
+    'submit',
+    edit_renderers=[submit_renderer],
+    display_renderers=[empty_display_renderer])
+
 factory.doc['widget']['submit'] = """\
 Submit tag inside the form
 """
+
 factory.doc['props']['submit.label'] = """\
 Label of the submit.
 """
@@ -809,8 +944,6 @@ factory.doc['props']['text.disabled'] = \
 """Flag  input field is disabled.
 """
 
-factory.register('submit', [], [submit_renderer])
-
 
 ###############################################################################
 # email
@@ -826,27 +959,22 @@ def email_extractor(widget, data):
     return val
 
 
-def email_renderer(widget, data):
-    # XXX: remove. Controlled form.novalidate instead
-    if not widget.attrs['html5type']:
-        widget.attrs['type'] = 'text'
-    return input_generic_renderer(widget, data)
-
+factory.register(
+    'email', 
+    extractors=[generic_extractor, generic_required_extractor, email_extractor],
+    edit_renderers=[input_generic_renderer])
 
 factory.doc['widget']['email'] = \
 """E-mail (HTML5) input widget.
 """
 
 factory.defaults['email.type'] = 'email'
-# XXX: remove. Controlled form.novalidate instead
-factory.defaults['email.html5type'] = True
+
 factory.defaults['email.default'] = ''
+
 factory.defaults['email.required_class'] = 'required'
+
 factory.defaults['email.class'] = 'email'
-factory.register('email', 
-                 [generic_extractor, generic_required_extractor,
-                  email_extractor],
-                 [email_renderer])
 
 
 ###############################################################################
@@ -864,34 +992,44 @@ def url_extractor(widget, data):
     return val
 
 
+factory.register(
+    'url',
+    extractors=[generic_extractor, generic_required_extractor, url_extractor],
+    edit_renderers=[input_generic_renderer])
+
 factory.doc['widget']['url'] = \
 """URL aka web address (HTML5) input widget.
 """
 
 factory.defaults['url.type'] = 'url'
+
 factory.defaults['url.default'] = ''
+
 factory.defaults['url.required_class'] = 'required'
+
 factory.defaults['url.class'] = 'url'
-factory.register('url',
-                 [generic_extractor, generic_required_extractor,
-                  url_extractor],
-                 [input_generic_renderer])
 
 
 ###############################################################################
 # search
 ###############################################################################
 
+factory.register(
+    'search', 
+    extractors=[generic_extractor, generic_required_extractor],
+    edit_renderers=[input_generic_renderer])
+
 factory.doc['widget']['search'] = """\
 Search widget (HTML5).
 """
+
 factory.defaults['search.type'] = 'search'
+
 factory.defaults['search.default'] = ''
+
 factory.defaults['search.required_class'] = 'required'
+
 factory.defaults['search.class'] = 'search'
-factory.register('search', 
-                 [generic_extractor, generic_required_extractor],
-                 [input_generic_renderer])
 
 
 ###############################################################################
@@ -926,10 +1064,18 @@ def number_extractor(widget, data):
     return val
 
 
+factory.register(
+    'number', 
+    extractors=[generic_extractor, generic_required_extractor,
+                number_extractor],
+    edit_renderers=[input_generic_renderer])
+
 factory.doc['widget']['number'] = """\
 Number widget (HTML5).
 """
+
 factory.defaults['number.type'] = 'number'
+
 factory.defaults['number.datatype'] = 'float'
 factory.doc['props']['number.datatype'] = """\
 Output datatype, one out of ``integer`` or ``float``.
@@ -938,14 +1084,14 @@ Output datatype, one out of ``integer`` or ``float``.
 factory.defaults['number.default'] = ''
 
 factory.defaults['number.min'] = None
+
 factory.defaults['number.max'] = None
+
 factory.defaults['number.step'] = None
+
 factory.defaults['number.required_class'] = 'required'
+
 factory.defaults['number.class'] = 'number'
-factory.register('number', 
-                 [generic_extractor, generic_required_extractor,
-                  number_extractor],
-                 [input_generic_renderer])
 
 
 ###############################################################################
@@ -957,18 +1103,18 @@ factory.register('number',
 def label_renderer(widget, data):
     tag = data.tag
     label_text = widget.attrs.get('label', widget.__name__)
-    for_path = widget.attrs['for']
-    if for_path:
-        for_widget = widget.root
-        for name in for_path.split('.'):
-            for_widget = for_widget[name]
-        for_ = cssid(for_widget, 'input')
-    else:
-        for_ = cssid(widget, 'input')
     label_attrs = {
-        'for_': for_,
         'class_': cssclasses(widget, data)
     }
+    if data.mode == 'edit':
+        for_path = widget.attrs['for']
+        if for_path:
+            for_widget = widget.root
+            for name in for_path.split('.'):
+                for_widget = for_widget[name]
+            label_attrs['for_'] = cssid(for_widget, 'input')
+        else:
+            label_attrs['for_'] = cssid(widget, 'input')
     help = u''
     if widget.attrs['help']:
         help_attrs = {'class_': widget.attrs['help_class']}
@@ -982,9 +1128,15 @@ def label_renderer(widget, data):
     return tag('label', label_text, help, **label_attrs) + rendered
 
 
+factory.register(
+    'label',
+    edit_renderers=[label_renderer],
+    display_renderers=[label_renderer])
+
 factory.doc['widget']['label'] = """\
 Label widget.
 """
+
 factory.defaults['label.position'] = 'before'
 factory.doc['props']['label.position'] = """\
 Label can be rendered at 3 different positions: ``before`` or ``after`` the 
@@ -995,18 +1147,19 @@ the label tag.
 factory.doc['props']['label.label'] = """\
 Text to be displayed as a label.
 """
+
+factory.defaults['label.for'] = None
 factory.doc['props']['label.for'] = """\
 Optional dottedpath of widget to be labled
 """
+
+factory.defaults['label.help'] = None
 factory.doc['props']['label.help'] = """\
 Optional help text (alternative description) to be rendered inside a div after
 the label.
 """
 
-factory.defaults['label.for'] = None
-factory.defaults['label.help'] = None
 factory.defaults['label.help_class'] = 'help'
-factory.register('label', [], [label_renderer])
 
 
 ###############################################################################
@@ -1025,16 +1178,22 @@ def field_renderer(widget, data):
     return tag('div', data.rendered, **div_attrs)
 
 
+factory.register(
+    'field',
+    edit_renderers=[field_renderer],
+    display_renderers=[field_renderer])
+
 factory.doc['widget']['field'] = """\
 Renders a div with an class field around the prior rendered output. This is 
 supposed to be used for styling and grouping purposes.
 """
+
 factory.defaults['field.class'] = 'field'
+
 factory.defaults['field.witherror'] = None
 factory.doc['props']['field.witherror'] = """\
 Put the class given with this property on the div if an error happened.
 """
-factory.register('field', [], [field_renderer])
 
 
 ###############################################################################
@@ -1052,9 +1211,15 @@ def error_renderer(widget, data):
     return tag('div', msgs, data.rendered, class_=cssclasses(widget, data))
 
 
+factory.register(
+    'error',
+    edit_renderers=[error_renderer],
+    display_renderers=[empty_display_renderer])
+
 factory.doc['widget']['error'] = """\
 Renders a div with an errormessage around the prior rendered output.
 """
+
 factory.defaults['error.error_class'] = 'error'
-factory.defaults['error.message_class'] = 'errormessage'
-factory.register('error', [], [error_renderer])
\ No newline at end of file
+
+factory.defaults['error.message_class'] = 'errormessage'
\ No newline at end of file
